@@ -270,7 +270,11 @@ func (h *BufPane) SetID(i uint64) {
 }
 
 func (h *BufPane) Name() string {
-	return h.Buf.GetName()
+	n := h.Buf.GetName()
+	if h.Buf.Modified() {
+		n += " +"
+	}
+	return n
 }
 
 // HandleEvent executes the tcell event properly
@@ -339,10 +343,11 @@ func (h *BufPane) HandleEvent(event tcell.Event) {
 				// release the mouse
 
 				// if !h.doubleClick && !h.tripleClick {
-				// 	h.Cursor.Loc = mouseLoc
 				// 	h.Cursor.SetSelectionEnd(h.Cursor.Loc)
-				// 	h.Cursor.CopySelection("primary")
 				// }
+				if h.Cursor.HasSelection() {
+					h.Cursor.CopySelection("primary")
+				}
 				h.mouseReleased = true
 			}
 		}
@@ -458,6 +463,7 @@ func (h *BufPane) DoRuneInsert(r rune) {
 		if recording_macro {
 			curmacro = append(curmacro, r)
 		}
+		h.Relocate()
 		h.PluginCBRune("onRune", r)
 	}
 }
@@ -511,109 +517,112 @@ func (h *BufPane) SetActive(b bool) {
 
 // BufKeyActions contains the list of all possible key actions the bufhandler could execute
 var BufKeyActions = map[string]BufKeyAction{
-	"CursorUp":               (*BufPane).CursorUp,
-	"CursorDown":             (*BufPane).CursorDown,
-	"CursorPageUp":           (*BufPane).CursorPageUp,
-	"CursorPageDown":         (*BufPane).CursorPageDown,
-	"CursorLeft":             (*BufPane).CursorLeft,
-	"CursorRight":            (*BufPane).CursorRight,
-	"CursorStart":            (*BufPane).CursorStart,
-	"CursorEnd":              (*BufPane).CursorEnd,
-	"SelectToStart":          (*BufPane).SelectToStart,
-	"SelectToEnd":            (*BufPane).SelectToEnd,
-	"SelectUp":               (*BufPane).SelectUp,
-	"SelectDown":             (*BufPane).SelectDown,
-	"SelectLeft":             (*BufPane).SelectLeft,
-	"SelectRight":            (*BufPane).SelectRight,
-	"WordRight":              (*BufPane).WordRight,
-	"WordLeft":               (*BufPane).WordLeft,
-	"SelectWordRight":        (*BufPane).SelectWordRight,
-	"SelectWordLeft":         (*BufPane).SelectWordLeft,
-	"DeleteWordRight":        (*BufPane).DeleteWordRight,
-	"DeleteWordLeft":         (*BufPane).DeleteWordLeft,
-	"SelectLine":             (*BufPane).SelectLine,
-	"SelectToStartOfLine":    (*BufPane).SelectToStartOfLine,
-	"SelectToStartOfText":    (*BufPane).SelectToStartOfText,
-	"SelectToEndOfLine":      (*BufPane).SelectToEndOfLine,
-	"ParagraphPrevious":      (*BufPane).ParagraphPrevious,
-	"ParagraphNext":          (*BufPane).ParagraphNext,
-	"InsertNewline":          (*BufPane).InsertNewline,
-	"Backspace":              (*BufPane).Backspace,
-	"Delete":                 (*BufPane).Delete,
-	"InsertTab":              (*BufPane).InsertTab,
-	"Save":                   (*BufPane).Save,
-	"SaveAll":                (*BufPane).SaveAll,
-	"SaveAs":                 (*BufPane).SaveAs,
-	"Find":                   (*BufPane).Find,
-	"FindNext":               (*BufPane).FindNext,
-	"FindPrevious":           (*BufPane).FindPrevious,
-	"Center":                 (*BufPane).Center,
-	"Undo":                   (*BufPane).Undo,
-	"Redo":                   (*BufPane).Redo,
-	"Copy":                   (*BufPane).Copy,
-	"Cut":                    (*BufPane).Cut,
-	"CutLine":                (*BufPane).CutLine,
-	"DuplicateLine":          (*BufPane).DuplicateLine,
-	"DeleteLine":             (*BufPane).DeleteLine,
-	"MoveLinesUp":            (*BufPane).MoveLinesUp,
-	"MoveLinesDown":          (*BufPane).MoveLinesDown,
-	"IndentSelection":        (*BufPane).IndentSelection,
-	"OutdentSelection":       (*BufPane).OutdentSelection,
-	"Autocomplete":           (*BufPane).Autocomplete,
-	"CycleAutocompleteBack":  (*BufPane).CycleAutocompleteBack,
-	"OutdentLine":            (*BufPane).OutdentLine,
-	"IndentLine":             (*BufPane).IndentLine,
-	"Paste":                  (*BufPane).Paste,
-	"PastePrimary":           (*BufPane).PastePrimary,
-	"SelectAll":              (*BufPane).SelectAll,
-	"OpenFile":               (*BufPane).OpenFile,
-	"Start":                  (*BufPane).Start,
-	"End":                    (*BufPane).End,
-	"PageUp":                 (*BufPane).PageUp,
-	"PageDown":               (*BufPane).PageDown,
-	"SelectPageUp":           (*BufPane).SelectPageUp,
-	"SelectPageDown":         (*BufPane).SelectPageDown,
-	"HalfPageUp":             (*BufPane).HalfPageUp,
-	"HalfPageDown":           (*BufPane).HalfPageDown,
-	"StartOfText":            (*BufPane).StartOfText,
-	"StartOfLine":            (*BufPane).StartOfLine,
-	"EndOfLine":              (*BufPane).EndOfLine,
-	"ToggleHelp":             (*BufPane).ToggleHelp,
-	"ToggleKeyMenu":          (*BufPane).ToggleKeyMenu,
-	"ToggleDiffGutter":       (*BufPane).ToggleDiffGutter,
-	"ToggleRuler":            (*BufPane).ToggleRuler,
-	"ClearStatus":            (*BufPane).ClearStatus,
-	"ShellMode":              (*BufPane).ShellMode,
-	"CommandMode":            (*BufPane).CommandMode,
-	"ToggleOverwriteMode":    (*BufPane).ToggleOverwriteMode,
-	"Escape":                 (*BufPane).Escape,
-	"Quit":                   (*BufPane).Quit,
-	"QuitAll":                (*BufPane).QuitAll,
-	"AddTab":                 (*BufPane).AddTab,
-	"PreviousTab":            (*BufPane).PreviousTab,
-	"NextTab":                (*BufPane).NextTab,
-	"NextSplit":              (*BufPane).NextSplit,
-	"PreviousSplit":          (*BufPane).PreviousSplit,
-	"Unsplit":                (*BufPane).Unsplit,
-	"VSplit":                 (*BufPane).VSplitAction,
-	"HSplit":                 (*BufPane).HSplitAction,
-	"ToggleMacro":            (*BufPane).ToggleMacro,
-	"PlayMacro":              (*BufPane).PlayMacro,
-	"Suspend":                (*BufPane).Suspend,
-	"ScrollUp":               (*BufPane).ScrollUpAction,
-	"ScrollDown":             (*BufPane).ScrollDownAction,
-	"SpawnMultiCursor":       (*BufPane).SpawnMultiCursor,
-	"SpawnMultiCursorUp":     (*BufPane).SpawnMultiCursorUp,
-	"SpawnMultiCursorDown":   (*BufPane).SpawnMultiCursorDown,
-	"SpawnMultiCursorSelect": (*BufPane).SpawnMultiCursorSelect,
-	"RemoveMultiCursor":      (*BufPane).RemoveMultiCursor,
-	"RemoveAllMultiCursors":  (*BufPane).RemoveAllMultiCursors,
-	"SkipMultiCursor":        (*BufPane).SkipMultiCursor,
-	"JumpToMatchingBrace":    (*BufPane).JumpToMatchingBrace,
-	"None":                   (*BufPane).None,
+	"CursorUp":                 (*BufPane).CursorUp,
+	"CursorDown":               (*BufPane).CursorDown,
+	"CursorPageUp":             (*BufPane).CursorPageUp,
+	"CursorPageDown":           (*BufPane).CursorPageDown,
+	"CursorLeft":               (*BufPane).CursorLeft,
+	"CursorRight":              (*BufPane).CursorRight,
+	"CursorStart":              (*BufPane).CursorStart,
+	"CursorEnd":                (*BufPane).CursorEnd,
+	"SelectToStart":            (*BufPane).SelectToStart,
+	"SelectToEnd":              (*BufPane).SelectToEnd,
+	"SelectUp":                 (*BufPane).SelectUp,
+	"SelectDown":               (*BufPane).SelectDown,
+	"SelectLeft":               (*BufPane).SelectLeft,
+	"SelectRight":              (*BufPane).SelectRight,
+	"WordRight":                (*BufPane).WordRight,
+	"WordLeft":                 (*BufPane).WordLeft,
+	"SelectWordRight":          (*BufPane).SelectWordRight,
+	"SelectWordLeft":           (*BufPane).SelectWordLeft,
+	"DeleteWordRight":          (*BufPane).DeleteWordRight,
+	"DeleteWordLeft":           (*BufPane).DeleteWordLeft,
+	"SelectLine":               (*BufPane).SelectLine,
+	"SelectToStartOfLine":      (*BufPane).SelectToStartOfLine,
+	"SelectToStartOfText":      (*BufPane).SelectToStartOfText,
+	"SelectToStartOfTextToggle":(*BufPane).SelectToStartOfTextToggle,
+	"SelectToEndOfLine":        (*BufPane).SelectToEndOfLine,
+	"ParagraphPrevious":        (*BufPane).ParagraphPrevious,
+	"ParagraphNext":            (*BufPane).ParagraphNext,
+	"InsertNewline":            (*BufPane).InsertNewline,
+	"Backspace":                (*BufPane).Backspace,
+	"Delete":                   (*BufPane).Delete,
+	"InsertTab":                (*BufPane).InsertTab,
+	"Save":                     (*BufPane).Save,
+	"SaveAll":                  (*BufPane).SaveAll,
+	"SaveAs":                   (*BufPane).SaveAs,
+	"Find":                     (*BufPane).Find,
+	"FindNext":                 (*BufPane).FindNext,
+	"FindPrevious":             (*BufPane).FindPrevious,
+	"Center":                   (*BufPane).Center,
+	"Undo":                     (*BufPane).Undo,
+	"Redo":                     (*BufPane).Redo,
+	"Copy":                     (*BufPane).Copy,
+	"Cut":                      (*BufPane).Cut,
+	"CutLine":                  (*BufPane).CutLine,
+	"DuplicateLine":            (*BufPane).DuplicateLine,
+	"DeleteLine":               (*BufPane).DeleteLine,
+	"MoveLinesUp":              (*BufPane).MoveLinesUp,
+	"MoveLinesDown":            (*BufPane).MoveLinesDown,
+	"IndentSelection":          (*BufPane).IndentSelection,
+	"OutdentSelection":         (*BufPane).OutdentSelection,
+	"Autocomplete":             (*BufPane).Autocomplete,
+	"CycleAutocompleteBack":    (*BufPane).CycleAutocompleteBack,
+	"OutdentLine":              (*BufPane).OutdentLine,
+	"IndentLine":               (*BufPane).IndentLine,
+	"Paste":                    (*BufPane).Paste,
+	"PastePrimary":             (*BufPane).PastePrimary,
+	"SelectAll":                (*BufPane).SelectAll,
+	"OpenFile":                 (*BufPane).OpenFile,
+	"Start":                    (*BufPane).Start,
+	"End":                      (*BufPane).End,
+	"PageUp":                   (*BufPane).PageUp,
+	"PageDown":                 (*BufPane).PageDown,
+	"SelectPageUp":             (*BufPane).SelectPageUp,
+	"SelectPageDown":           (*BufPane).SelectPageDown,
+	"HalfPageUp":               (*BufPane).HalfPageUp,
+	"HalfPageDown":             (*BufPane).HalfPageDown,
+	"StartOfText":              (*BufPane).StartOfText,
+	"StartOfTextToggle":        (*BufPane).StartOfTextToggle,
+	"StartOfLine":              (*BufPane).StartOfLine,
+	"EndOfLine":                (*BufPane).EndOfLine,
+	"ToggleHelp":               (*BufPane).ToggleHelp,
+	"ToggleKeyMenu":            (*BufPane).ToggleKeyMenu,
+	"ToggleDiffGutter":         (*BufPane).ToggleDiffGutter,
+	"ToggleRuler":              (*BufPane).ToggleRuler,
+	"ClearStatus":              (*BufPane).ClearStatus,
+	"ShellMode":                (*BufPane).ShellMode,
+	"CommandMode":              (*BufPane).CommandMode,
+	"ToggleOverwriteMode":      (*BufPane).ToggleOverwriteMode,
+	"Escape":                   (*BufPane).Escape,
+	"Quit":                     (*BufPane).Quit,
+	"QuitAll":                  (*BufPane).QuitAll,
+	"AddTab":                   (*BufPane).AddTab,
+	"PreviousTab":              (*BufPane).PreviousTab,
+	"NextTab":                  (*BufPane).NextTab,
+	"NextSplit":                (*BufPane).NextSplit,
+	"PreviousSplit":            (*BufPane).PreviousSplit,
+	"Unsplit":                  (*BufPane).Unsplit,
+	"VSplit":                   (*BufPane).VSplitAction,
+	"HSplit":                   (*BufPane).HSplitAction,
+	"ToggleMacro":              (*BufPane).ToggleMacro,
+	"PlayMacro":                (*BufPane).PlayMacro,
+	"Suspend":                  (*BufPane).Suspend,
+	"ScrollUp":                 (*BufPane).ScrollUpAction,
+	"ScrollDown":               (*BufPane).ScrollDownAction,
+	"SpawnMultiCursor":         (*BufPane).SpawnMultiCursor,
+	"SpawnMultiCursorUp":       (*BufPane).SpawnMultiCursorUp,
+	"SpawnMultiCursorDown":     (*BufPane).SpawnMultiCursorDown,
+	"SpawnMultiCursorSelect":   (*BufPane).SpawnMultiCursorSelect,
+	"RemoveMultiCursor":        (*BufPane).RemoveMultiCursor,
+	"RemoveAllMultiCursors":    (*BufPane).RemoveAllMultiCursors,
+	"SkipMultiCursor":          (*BufPane).SkipMultiCursor,
+	"JumpToMatchingBrace":      (*BufPane).JumpToMatchingBrace,
+	"JumpLine":                 (*BufPane).JumpLine,
+	"None":                     (*BufPane).None,
 
 	// This was changed to InsertNewline but I don't want to break backwards compatibility
-	"InsertEnter": (*BufPane).InsertNewline,
+	"InsertEnter":              (*BufPane).InsertNewline,
 }
 
 // BufMouseActions contains the list of all possible mouse actions the bufhandler could execute
@@ -627,54 +636,56 @@ var BufMouseActions = map[string]BufMouseAction{
 // Generally actions that modify global editor state like quitting or
 // saving should not be included in this list
 var MultiActions = map[string]bool{
-	"CursorUp":            true,
-	"CursorDown":          true,
-	"CursorPageUp":        true,
-	"CursorPageDown":      true,
-	"CursorLeft":          true,
-	"CursorRight":         true,
-	"CursorStart":         true,
-	"CursorEnd":           true,
-	"SelectToStart":       true,
-	"SelectToEnd":         true,
-	"SelectUp":            true,
-	"SelectDown":          true,
-	"SelectLeft":          true,
-	"SelectRight":         true,
-	"WordRight":           true,
-	"WordLeft":            true,
-	"SelectWordRight":     true,
-	"SelectWordLeft":      true,
-	"DeleteWordRight":     true,
-	"DeleteWordLeft":      true,
-	"SelectLine":          true,
-	"SelectToStartOfLine": true,
-	"SelectToStartOfText": true,
-	"SelectToEndOfLine":   true,
-	"ParagraphPrevious":   true,
-	"ParagraphNext":       true,
-	"InsertNewline":       true,
-	"Backspace":           true,
-	"Delete":              true,
-	"InsertTab":           true,
-	"FindNext":            true,
-	"FindPrevious":        true,
-	"Cut":                 true,
-	"CutLine":             true,
-	"DuplicateLine":       true,
-	"DeleteLine":          true,
-	"MoveLinesUp":         true,
-	"MoveLinesDown":       true,
-	"IndentSelection":     true,
-	"OutdentSelection":    true,
-	"OutdentLine":         true,
-	"IndentLine":          true,
-	"Paste":               true,
-	"PastePrimary":        true,
-	"SelectPageUp":        true,
-	"SelectPageDown":      true,
-	"StartOfLine":         true,
-	"StartOfText":         true,
-	"EndOfLine":           true,
-	"JumpToMatchingBrace": true,
+	"CursorUp":                  true,
+	"CursorDown":                true,
+	"CursorPageUp":              true,
+	"CursorPageDown":            true,
+	"CursorLeft":                true,
+	"CursorRight":               true,
+	"CursorStart":               true,
+	"CursorEnd":                 true,
+	"SelectToStart":             true,
+	"SelectToEnd":               true,
+	"SelectUp":                  true,
+	"SelectDown":                true,
+	"SelectLeft":                true,
+	"SelectRight":               true,
+	"WordRight":                 true,
+	"WordLeft":                  true,
+	"SelectWordRight":           true,
+	"SelectWordLeft":            true,
+	"DeleteWordRight":           true,
+	"DeleteWordLeft":            true,
+	"SelectLine":                true,
+	"SelectToStartOfLine":       true,
+	"SelectToStartOfText":       true,
+	"SelectToStartOfTextToggle": true,
+	"SelectToEndOfLine":         true,
+	"ParagraphPrevious":         true,
+	"ParagraphNext":             true,
+	"InsertNewline":             true,
+	"Backspace":                 true,
+	"Delete":                    true,
+	"InsertTab":                 true,
+	"FindNext":                  true,
+	"FindPrevious":              true,
+	"Cut":                       true,
+	"CutLine":                   true,
+	"DuplicateLine":             true,
+	"DeleteLine":                true,
+	"MoveLinesUp":               true,
+	"MoveLinesDown":             true,
+	"IndentSelection":           true,
+	"OutdentSelection":          true,
+	"OutdentLine":               true,
+	"IndentLine":                true,
+	"Paste":                     true,
+	"PastePrimary":              true,
+	"SelectPageUp":              true,
+	"SelectPageDown":            true,
+	"StartOfLine":               true,
+	"StartOfText":               true,
+	"StartOfTextToggle":         true,
+	"EndOfLine":                 true,
+	"JumpToMatchingBrace":       true,
 }
