@@ -732,23 +732,23 @@ func (h *BufPane) ReplaceCmd(args []string) {
 
 	nreplaced := 0
 	start := h.Buf.Start()
-	// end := h.Buf.End()
-	// if h.Cursor.HasSelection() {
-	// 	start = h.Cursor.CurSelection[0]
-	// 	end = h.Cursor.CurSelection[1]
-	// }
+	end := h.Buf.End()
+	selection := h.Cursor.HasSelection()
+	if selection {
+		start = h.Cursor.CurSelection[0]
+		end = h.Cursor.CurSelection[1]
+	}
 	if all {
-		nreplaced = h.Buf.ReplaceRegex(start, h.Buf.End(), regex, replace)
+		nreplaced, _ = h.Buf.ReplaceRegex(start, end, regex, replace)
 	} else {
 		inRange := func(l buffer.Loc) bool {
-			return l.GreaterEqual(start) && l.LessEqual(h.Buf.End())
+			return l.GreaterEqual(start) && l.LessEqual(end)
 		}
 
 		searchLoc := start
-		searching := true
 		var doReplacement func()
 		doReplacement = func() {
-			locs, found, err := h.Buf.FindNext(search, start, h.Buf.End(), searchLoc, true, !noRegex)
+			locs, found, err := h.Buf.FindNext(search, start, end, searchLoc, true, !noRegex)
 			if err != nil {
 				InfoBar.Error(err)
 				return
@@ -762,12 +762,15 @@ func (h *BufPane) ReplaceCmd(args []string) {
 			h.Cursor.SetSelectionStart(locs[0])
 			h.Cursor.SetSelectionEnd(locs[1])
 
+			h.Relocate()
+
 			InfoBar.YNPrompt("Perform replacement (y,n,esc)", func(yes, canceled bool) {
 				if !canceled && yes {
-					h.Buf.Replace(locs[0], locs[1], replaceStr)
+					_, nrunes := h.Buf.ReplaceRegex(locs[0], locs[1], regex, replace)
 
 					searchLoc = locs[0]
-					searchLoc.X += utf8.RuneCount(replace)
+					searchLoc.X += nrunes + locs[0].Diff(locs[1], h.Buf)
+					end.Move(nrunes, h.Buf)
 					h.Cursor.Loc = searchLoc
 					nreplaced++
 				} else if !canceled && !yes {
@@ -778,23 +781,29 @@ func (h *BufPane) ReplaceCmd(args []string) {
 					h.Buf.RelocateCursors()
 					return
 				}
-				if searching {
-					doReplacement()
-				}
+				doReplacement()
 			})
 		}
 		doReplacement()
 	}
 
 	h.Buf.RelocateCursors()
+	h.Relocate()
 
+	var s string
 	if nreplaced > 1 {
-		InfoBar.Message("Replaced ", nreplaced, " occurrences of ", search)
+		s = fmt.Sprintf("Replaced %d occurrences of %s", nreplaced, search)
 	} else if nreplaced == 1 {
-		InfoBar.Message("Replaced ", nreplaced, " occurrence of ", search)
+		s = fmt.Sprintf("Replaced 1 occurrence of %s", search)
 	} else {
-		InfoBar.Message("Nothing matched ", search)
+		s = fmt.Sprintf("Nothing matched %s", search)
 	}
+
+	if selection {
+		s += " in selection"
+	}
+
+	InfoBar.Message(s)
 }
 
 // ReplaceAllCmd replaces search term all at once
@@ -823,7 +832,11 @@ func (h *BufPane) TermCmd(args []string) {
 
 	term := func(i int, newtab bool) {
 		t := new(shell.Terminal)
-		t.Start(args, false, true, nil, nil)
+		err := t.Start(args, false, true, nil, nil)
+		if err != nil {
+			InfoBar.Error(err)
+			return
+		}
 
 		id := h.ID()
 		if newtab {
