@@ -17,10 +17,10 @@ func DefaultBindings() map[string]string {
 		"AltDown":        "MoveLinesDown",
 		"AltShiftRight":  "SelectWordRight",
 		"AltShiftLeft":   "SelectWordLeft",
-		"CtrlLeft":       "StartOfText",
+		"CtrlLeft":       "StartOfTextToggle",
 		"CtrlRight":      "EndOfLine",
-		"CtrlShiftLeft":  "SelectToStartOfText",
-		"ShiftHome":      "SelectToStartOfText",
+		"CtrlShiftLeft":  "SelectToStartOfTextToggle",
+		"ShiftHome":      "SelectToStartOfTextToggle",
 		"CtrlShiftRight": "SelectToEndOfLine",
 		"ShiftEnd":       "SelectToEndOfLine",
 		"CtrlUp":         "CursorStart",
@@ -52,7 +52,7 @@ func DefaultBindings() map[string]string {
 		"CtrlT":          "AddTab",
 		"Alt,":           "PreviousTab",
 		"Alt.":           "NextTab",
-		"Home":           "StartOfText",
+		"Home":           "StartOfTextToggle",
 		"End":            "EndOfLine",
 		"CtrlHome":       "CursorStart",
 		"CtrlEnd":        "CursorEnd",
