@@ -136,12 +136,12 @@ func (w *BufWindow) Relocate() bool {
 	if w.drawStatus {
 		h--
 	}
-	if b.LinesNum() <= h {
-		height = w.Height
-	}
 	ret := false
 	activeC := w.Buf.GetActiveCursor()
 	cy := activeC.Y
+	if activeC.HasSelection() {
+		cy = activeC.CurSelection[0].Y
+	}
 	scrollmargin := int(b.Settings["scrollmargin"].(float64))
 	if cy < w.StartLine+scrollmargin && cy > scrollmargin-1 {
 		w.StartLine = cy - scrollmargin
@@ -275,14 +275,7 @@ func (w *BufWindow) LocFromVisual(svloc buffer.Loc) buffer.Loc {
 					if vloc.Y >= bufHeight {
 						break
 					}
-					vloc.X = 0
-					if b.Settings["diffgutter"].(bool) {
-						vloc.X++
-					}
-					// This will draw an empty line number because the current line is wrapped
-					if b.Settings["ruler"].(bool) {
-						vloc.X += maxLineNumLength + 1
-					}
+					vloc.X = w.gutterOffset
 				}
 			}
 		}
@@ -407,21 +400,7 @@ func (w *BufWindow) displayBuffer() {
 		bufWidth--
 	}
 
-	if len(b.Modifications) > 0 {
-		if b.Settings["syntax"].(bool) && b.SyntaxDef != nil {
-			for _, r := range b.Modifications {
-				rx := util.Clamp(r.X, 0, b.LinesNum())
-				ry := util.Clamp(r.Y, 0, b.LinesNum())
-				final := -1
-				for i := rx; i <= ry; i++ {
-					final = util.Max(b.Highlighter.ReHighlightStates(b, i), final)
-				}
-				b.Highlighter.HighlightMatches(b, rx, final+1)
-			}
-		}
-
-		b.ClearModifications()
-
+	if b.ModifiedThisFrame {
 		if b.Settings["diffgutter"].(bool) {
 			b.UpdateDiff(func(synchronous bool) {
 				// If the diff was updated asynchronously, the outer call to
@@ -436,6 +415,7 @@ func (w *BufWindow) displayBuffer() {
 				}
 			})
 		}
+		b.ModifiedThisFrame = false
 	}
 
 	var matchingBraces []buffer.Loc
@@ -579,7 +559,7 @@ func (w *BufWindow) displayBuffer() {
 				}
 
 				if s, ok := config.Colorscheme["color-column"]; ok {
-					if colorcolumn != 0 && vloc.X-w.gutterOffset == colorcolumn {
+					if colorcolumn != 0 && vloc.X-w.gutterOffset+w.StartCol == colorcolumn {
 						fg, _, _ := s.Decompose()
 						style = style.Background(fg)
 					}
@@ -645,9 +625,13 @@ func (w *BufWindow) displayBuffer() {
 						break
 					}
 					vloc.X = 0
+					if hasMessage {
+						w.drawGutter(&vloc, &bloc)
+					}
 					if b.Settings["diffgutter"].(bool) {
 						w.drawDiffGutter(lineNumStyle, true, &vloc, &bloc)
 					}
+
 					// This will draw an empty line number because the current line is wrapped
 					if b.Settings["ruler"].(bool) {
 						w.drawLineNum(lineNumStyle, true, maxLineNumLength, &vloc, &bloc)
@@ -669,7 +653,7 @@ func (w *BufWindow) displayBuffer() {
 		for i := vloc.X; i < bufWidth; i++ {
 			curStyle := style
 			if s, ok := config.Colorscheme["color-column"]; ok {
-				if colorcolumn != 0 && i-w.gutterOffset == colorcolumn {
+				if colorcolumn != 0 && i-w.gutterOffset+w.StartCol == colorcolumn {
 					fg, _, _ := s.Decompose()
 					curStyle = style.Background(fg)
 				}
