@@ -136,6 +136,12 @@ func Init() {
 		os.Setenv("TCELL_TRUECOLOR", "disable")
 	}
 
+	var oldTerm string
+	if config.GetGlobalOption("xterm").(bool) {
+		oldTerm = os.Getenv("TERM")
+		os.Setenv("TERM", "xterm-256color")
+	}
+
 	// Initilize tcell
 	var err error
 	Screen, err = tcell.NewScreen()
@@ -149,6 +155,11 @@ func Init() {
 		os.Exit(1)
 	}
 
+	// restore TERM
+	if config.GetGlobalOption("xterm").(bool) {
+		os.Setenv("TERM", oldTerm)
+	}
+
 	if config.GetGlobalOption("mouse").(bool) {
 		Screen.EnableMouse()
 	}
